//
//  BigLetterView.swift
//  TypingTutor
//
//  Created by narita on 2019/02/06.
//  Copyright © 2019 narita. All rights reserved.
//

import Cocoa

class BigLetterView: NSView, NSServicesMenuRequestor
{
    // MARK: - class methods
    
    // MARK: - init methods
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    override func awakeFromNib()
    {
        // 呼び出し場所は適当でない。nib経由以外で作成される場合には、別の方法で呼び出すこと。
        NSApp.registerServicesMenuSendTypes([.string], returnTypes: [.string])
    }
    
    // MARK: - responder event methods

    // どのようなUTIを使うサービスを利用可能かを返す。
    override func validRequestor(forSendType sendType: NSPasteboard.PasteboardType?,
                                 returnType: NSPasteboard.PasteboardType?) -> Any?
    {
//        print("sendType \(sendType)")
//        print("returnType \(returnType)")

        // 編集可能か、現在選択されているかで、サービスの利用の可否を判定する。
        // このBigLetterViewでは常に選択されていて編集可能なので、.stringならば全部OKになっています。
        // なお、どちらもnilの場合は考えなくて良い

        if sendType == .string && returnType == nil
        {
            return self
        }
        
        if sendType == nil && returnType == .string
        {
            return self
        }

        if sendType == .string && returnType == .string
        {
            return self
        }
        
        return super.validRequestor(forSendType: sendType, returnType: returnType)
    }

    @objc func writeSelection(to pboard: NSPasteboard, types: [NSPasteboard.PasteboardType]) -> Bool
    {
        // なぜか、System側は、".string"でなくて、"NSStringPboardType"を返すので追加。info.plistが関係するかもしれない
        if types.contains(.string) || types.contains(NSPasteboard.PasteboardType(rawValue: "NSStringPboardType"))
        {
            self.writeToPasteboard(pboard)
            return true
        }
    
        return false
    }
    
    @objc func readSelection(from pboard: NSPasteboard) -> Bool
    {
        return self.readFromPasteboard(pboard)
    }

    
    override func resignFirstResponder() -> Bool
    {
        self.setKeyboardFocusRingNeedsDisplay(self.bounds)
        
        return true
    }
    
    override func becomeFirstResponder() -> Bool
    {
        self.needsDisplay = true
        
        return true
    }

    override func keyDown(with event: NSEvent)
    {
        self.interpretKeyEvents([event])
    }
    
    override func insertText(_ insertString: Any)
    {
        self.string = insertString as! NSString
    }
    
    override func  insertTab(_ sender: Any?)
    {
        self.window?.selectKeyView(following: self)
    }
    
    override func  insertBacktab(_ sender: Any?)
    {
        self.window?.selectKeyView(preceding: self)
    }

    override func  deleteBackward(_ sender: Any?)
    {
        self.string = ""
    }

    // MARK: - action methods
    
    func writeToPasteboard(_ pb: NSPasteboard )
    {
        pb.clearContents()
        
        let thePDFData = self.dataWithPDF(inside: self.bounds)
        let thePDFItem = NSPasteboardItem()
        thePDFItem.setData(thePDFData, forType: .pdf)
        
        let theTextItem = NSPasteboardItem()
        theTextItem.setString(self.string as String, forType: .string)
        
        pb.writeObjects([thePDFItem, theTextItem])
    }
    
    func readFromPasteboard(_ pb: NSPasteboard ) -> Bool
    {
        let classes = [NSString.self]
        let objects = pb.readObjects(forClasses: classes, options: nil)
        
        if objects!.count > 0
        {
            if let value = objects![0] as? NSString
            {
                // メンドくさいのでサービスから帰る文字列が1以外でもOKにしておく。
                //if value.length == 1
                //{
                    self.string = value
                    return true
                //}
            }
        }
        
        return false
    }
    
    @IBAction func cut(_ sender: Any?)
    {
        self.copy(sender)
        self.string = ""
    }

    @IBAction func copy(_ sender: Any?)
    {
        self.writeToPasteboard(NSPasteboard.general)
    }

    @IBAction func paste(_ sender: Any?)
    {
        if self.readFromPasteboard(NSPasteboard.general) == false
        {
            __NSBeep()
        }
    }



    // MARK: - drawing methods

    func drawStringCenteredIn(rect: NSRect)
    {
        let theStringSize: NSSize = self.string.size(withAttributes: (self.attributes as! [NSAttributedString.Key : Any]))
        
        let theStringOrigin = NSMakePoint(
                rect.origin.x + (rect.size.width - theStringSize.width) / 2,
                rect.origin.y + (rect.size.height - theStringSize.height) / 2)
        
        self.string.draw(at: theStringOrigin, withAttributes: (self.attributes as! [NSAttributedString.Key : Any]))
    }
    
    override func draw(_ dirtyRect: NSRect)
    {
        
        let  theBounds = self.bounds
        
        self.bgColor.set()
        NSBezierPath.fill(theBounds)
        self.drawStringCenteredIn(rect: theBounds)
        
        if(
                self.window?.firstResponder == self
            &&
                NSGraphicsContext.currentContextDrawingToScreen()
            )
        {
            NSGraphicsContext.saveGraphicsState()
            
            NSFocusRingPlacement.only.set()
            
            NSBezierPath.stroke(theBounds)
            
            NSGraphicsContext.restoreGraphicsState()
        }
    }

    
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    @objc var bgColor: NSColor = NSColor.yellow
    {
        didSet
        {
            self.needsDisplay = true
        }
    }
    
    @objc var string: NSString = ""
    {
        didSet
        {
            self.needsDisplay = true
        }
    }
    
    @objc lazy var attributes: NSMutableDictionary =
    {
        let shadow = NSShadow()
        shadow.shadowOffset = NSMakeSize(3, -3)
        
        return [
            NSAttributedString.Key.font : NSFont.userFont(ofSize: 75)!,
            NSAttributedString.Key.foregroundColor : NSColor.red,
            NSAttributedString.Key.shadow : shadow
        ]
    }()
    

    @objc var bold: Bool
    {
        set(newValue)
        {
            let theFontManager = NSFontManager.shared
            var theFont: NSFont = self.attributes.object(forKey:NSAttributedString.Key.font) as! NSFont
            
            if( newValue )
            {
                theFont = theFontManager.convert(theFont,
                                                 toHaveTrait: NSFontTraitMask.boldFontMask)
            }
            else
            {
                theFont = theFontManager.convert(theFont,
                                                 toNotHaveTrait: NSFontTraitMask.boldFontMask)
            }
            
            self.attributes.setObject(theFont, forKey: NSAttributedString.Key.font as NSCopying)
            self.needsDisplay = true
        }
        
        get
        {
            if
                NSFontManager.shared.traits(of: self.attributes.object(forKey: NSAttributedString.Key.font) as! NSFont ).rawValue & NSFontTraitMask.boldFontMask.rawValue != 0
            {
                return true
            }
            else
            {
                return false
            }
        }
    }
    
    @objc var italic: Bool
    {
        set(newValue)
        {
            let theFontManager = NSFontManager.shared
            var theFont: NSFont = self.attributes.object(forKey:NSAttributedString.Key.font) as! NSFont
            
            if( newValue )
            {
                theFont = theFontManager.convert(theFont,
                                                 toHaveTrait: NSFontTraitMask.italicFontMask)
            }
            else
            {
                theFont = theFontManager.convert(theFont,
                                                 toNotHaveTrait: NSFontTraitMask.italicFontMask)
            }
            
            self.attributes.setObject(theFont, forKey: NSAttributedString.Key.font as NSCopying)
            self.needsDisplay = true
        }
        
        get
        {
            if
                NSFontManager.shared.traits(of: self.attributes.object(forKey: NSAttributedString.Key.font) as! NSFont ).rawValue & NSFontTraitMask.italicFontMask.rawValue != 0
            {
                return true
            }
            else
            {
                return false
            }
        }
    }
    
    override var isOpaque: Bool
    {
        return true
    }

    override var acceptsFirstResponder: Bool
    {
        return true
    }
    
    
    // MARK: - Utility methods

}
